@@ -26,6 +26,7 @@
 #include "kmime_headers.h"
 #include "kmime_util.h"
 #include "kmime_util_p.h"
+#include "kmime_codecs.h"
 #include "kmime_dateformatter.h"
 #include "kmime_warning.h"
 
@@ -47,187 +48,6 @@ using namespace KMime::Types;
 namespace KMime
 {
 
-namespace Types
-{
-
-// QUrl::fromAce is extremely expensive, so only use it when necessary.
-// Fortunately, the presence of IDNA is readily detected with a substring match...
-static inline QString QUrl_fromAce_wrapper(const QString &domain)
-{
-    if (domain.contains(QLatin1String("xn--"))) {
-        return QUrl::fromAce(domain.toLatin1());
-    } else {
-        return domain;
-    }
-}
-
-static QString addr_spec_as_string(const AddrSpec &as, bool pretty)
-{
-    if (as.isEmpty()) {
-        return QString();
-    }
-
-    static const QChar dotChar = QLatin1Char('.');
-    static const QChar backslashChar = QLatin1Char('\\');
-    static const QChar quoteChar = QLatin1Char('"');
-
-    bool needsQuotes = false;
-    QString result;
-    result.reserve(as.localPart.length() + as.domain.length() + 1);
-    for (int i = 0 ; i < as.localPart.length() ; ++i) {
-        const QChar ch = as.localPart.at(i);
-        if (ch == dotChar || isAText(ch.toLatin1())) {
-            result += ch;
-        } else {
-            needsQuotes = true;
-            if (ch == backslashChar || ch == quoteChar) {
-                result += backslashChar;
-            }
-            result += ch;
-        }
-    }
-    const QString dom = pretty ? QUrl_fromAce_wrapper(as.domain) : as.domain ;
-    if (needsQuotes) {
-        result = quoteChar + result + quoteChar;
-    }
-    if (dom.isEmpty()) {
-        return result;
-    } else {
-        result += QLatin1Char('@');
-        result += dom;
-        return result;
-    }
-}
-
-QString AddrSpec::asString() const
-{
-    return addr_spec_as_string(*this, false);
-}
-
-QString AddrSpec::asPrettyString() const
-{
-    return addr_spec_as_string(*this, true);
-}
-
-bool AddrSpec::isEmpty() const
-{
-    return localPart.isEmpty() && domain.isEmpty();
-}
-
-QByteArray Mailbox::address() const
-{
-    QByteArray result;
-    const QString asString = addr_spec_as_string(mAddrSpec, false);
-    if (!asString.isEmpty()) {
-        result = asString.toLatin1();
-    }
-    return result;
-    //return mAddrSpec.asString().toLatin1();
-}
-
-AddrSpec Mailbox::addrSpec() const
-{
-    return mAddrSpec;
-}
-
-QString Mailbox::name() const
-{
-    return mDisplayName;
-}
-
-void Mailbox::setAddress(const AddrSpec &addr)
-{
-    mAddrSpec = addr;
-}
-
-void Mailbox::setAddress(const QByteArray &addr)
-{
-    const char *cursor = addr.constData();
-    if (!HeaderParsing::parseAngleAddr(cursor,
-                                       cursor + addr.length(), mAddrSpec)) {
-        if (!HeaderParsing::parseAddrSpec(cursor, cursor + addr.length(),
-                                          mAddrSpec)) {
-            qWarning() << "Invalid address";
-            return;
-        }
-    }
-}
-
-void Mailbox::setName(const QString &name)
-{
-    mDisplayName = removeBidiControlChars(name);
-}
-
-void Mailbox::setNameFrom7Bit(const QByteArray &name,
-                              const QByteArray &defaultCharset)
-{
-    QByteArray cs;
-    setName(decodeRFC2047String(name, cs, defaultCharset, false));
-}
-
-bool Mailbox::hasAddress() const
-{
-    return !mAddrSpec.isEmpty();
-}
-
-bool Mailbox::hasName() const
-{
-    return !mDisplayName.isEmpty();
-}
-
-QString Mailbox::prettyAddress() const
-{
-    return prettyAddress(QuoteNever);
-}
-
-QString Mailbox::prettyAddress(Quoting quoting) const
-{
-    if (!hasName()) {
-        return QLatin1String(address());
-    }
-    QString s = name();
-    if (quoting != QuoteNever) {
-        addQuotes(s, quoting == QuoteAlways /*bool force*/);
-    }
-
-    if (hasAddress()) {
-        s += QLatin1String(" <") + QLatin1String(address()) + QLatin1Char('>');
-    }
-    return s;
-}
-
-void Mailbox::fromUnicodeString(const QString &s)
-{
-    from7BitString(encodeRFC2047Sentence(s, "utf-8"));
-}
-
-void Mailbox::from7BitString(const QByteArray &s)
-{
-    const char *cursor = s.constData();
-    HeaderParsing::parseMailbox(cursor, cursor + s.length(), *this);
-}
-
-QByteArray KMime::Types::Mailbox::as7BitString(const QByteArray &encCharset) const
-{
-    if (!hasName()) {
-        return address();
-    }
-    QByteArray rv;
-    if (isUsAscii(name())) {
-        QByteArray tmp = name().toLatin1();
-        addQuotes(tmp, false);
-        rv += tmp;
-    } else {
-        rv += encodeRFC2047String(name(), encCharset, true);
-    }
-    if (hasAddress()) {
-        rv += " <" + address() + '>';
-    }
-    return rv;
-}
-
-} // namespace Types
-
 namespace HeaderParsing
 {
 
@@ -1381,10 +1201,6 @@ bool parseAddressList(const char *&scursor, const char *const send,
     return true;
 }
 
-static QString asterisk = QString::fromLatin1("*0*", 1);
-static QString asteriskZero = QString::fromLatin1("*0*", 2);
-//static QString asteriskZeroAsterisk = QString::fromLatin1( "*0*", 3 );
-
 // FIXME: Get rid of the very ugly "QStringOrQPair" thing. At this level, we are supposed to work
 //        on byte arrays, not strings! The result parameter should be a simple
 //        QPair<QByteArray,QByteArray>, which is the attribute name and the value.
@@ -1425,7 +1241,7 @@ bool parseParameter(const char *&scursor, const char *const send,
     eatCFWS(scursor, send, isCRLF);
     if (scursor == send) {
         // don't choke on attribute=, meaning the value was omitted:
-        if (maybeAttribute.endsWith(asterisk)) {
+        if (maybeAttribute.endsWith(QLatin1Char('*'))) {
             KMIME_WARN << "attribute ends with \"*\", but value is empty!"
                        "Chopping away \"*\".";
             maybeAttribute.truncate(maybeAttribute.length() - 1);
@@ -1443,7 +1259,7 @@ bool parseParameter(const char *&scursor, const char *const send,
     if (*scursor == '"') {
         // value is a quoted-string:
         scursor++;
-        if (maybeAttribute.endsWith(asterisk)) {
+        if (maybeAttribute.endsWith(QLatin1Char('*'))) {
             // attributes ending with "*" designate extended-parameters,
             // which cannot have quoted-strings as values. So we remove the
             // trailing "*" to not confuse upper layers.
@@ -1695,7 +1511,7 @@ bool parseParameterListWithCharset(const char *&scursor,
             EncodingMode encodingMode = NoEncoding;
 
             // is the value rfc2331-encoded?
-            if (attribute.endsWith(asterisk)) {
+            if (attribute.endsWith(QLatin1Char('*'))) {
                 attribute.truncate(attribute.length() - 1);
                 mode |= Encoded;
                 encodingMode = RFC2231;
@@ -1706,7 +1522,7 @@ bool parseParameterListWithCharset(const char *&scursor,
                 encodingMode = RFC2047;
             }
             // is the value continued?
-            if (attribute.endsWith(asteriskZero)) {
+            if (attribute.endsWith(QLatin1String("*0"))) {
                 attribute.truncate(attribute.length() - 2);
                 mode |= Continued;
             }
@@ -1719,7 +1535,7 @@ bool parseParameterListWithCharset(const char *&scursor,
                                        false, /* isn't continuation */
                                        value, (*it).qpair, charset);
                 } else if (encodingMode == RFC2047) {
-                    value += decodeRFC2047String((*it).qstring.toLatin1(), charset);
+                    value += KCodecs::decodeRFC2047String((*it).qstring.toLatin1(), &charset);
                 }
             } else {
                 // not encoded.
@@ -1746,7 +1562,7 @@ bool parseParameterListWithCharset(const char *&scursor,
             //
 
             // ignore the section and trust QMap to have sorted the keys:
-            if (it.key().endsWith(asterisk)) {
+            if (it.key().endsWith(QLatin1Char('*'))) {
                 // encoded
                 decodeRFC2231Value(rfc2231Codec, textcodec,
                                    true, /* is continuation */
@@ -1777,7 +1593,7 @@ bool parseParameterList(const char *&scursor, const char *const send,
     return parseParameterListWithCharset(scursor, send, result, charset, isCRLF);
 }
 
-static const char *const stdDayNames[] = {
+static const char stdDayNames[][4] = {
     "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 static const int stdDayNamesLen = sizeof stdDayNames / sizeof *stdDayNames;
@@ -1800,7 +1616,7 @@ static bool parseDayName(const char *&scursor, const char *const send)
     return false;
 }
 
-static const char *const stdMonthNames[] = {
+static const char stdMonthNames[][4] = {
     "Jan", "Feb", "Mar", "Apr", "May", "Jun",
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 };
@@ -1827,7 +1643,7 @@ static bool parseMonthName(const char *&scursor, const char *const send,
 }
 
 static const struct {
-    const char *tzName;
+    const char tzName[5];
     long int secsEastOfGMT;
 } timeZones[] = {
     // rfc 822 timezones:
@@ -2211,23 +2027,26 @@ Headers::Base *extractFirstHeader(QByteArray &head)
     Headers::Base *header = 0;
 
     int startOfFieldBody = head.indexOf(':');
-    const int endOfFieldHeader = startOfFieldBody;
 
     if (startOfFieldBody > -1) {      //there is another header
+        // Split the original data
+        head[startOfFieldBody] = '\0';
+        // rawType references the actual data from 'head'
+        QByteArray rawType = QByteArray::fromRawData(head.constData(), startOfFieldBody);
+
         startOfFieldBody++; //skip the ':'
         if (head[startOfFieldBody] == ' ') {   // skip the space after the ':', if there
             startOfFieldBody++;
         }
         endOfFieldBody = findHeaderLineEnd(head, startOfFieldBody, &folded);
-
-        QByteArray rawType = head.left(endOfFieldHeader);
-        QByteArray rawFieldBody = head.mid(startOfFieldBody, endOfFieldBody - startOfFieldBody);
+        // rawFieldBody references actual data from 'heaed'
+        QByteArray rawFieldBody = QByteArray::fromRawData(head.constData() + startOfFieldBody, endOfFieldBody - startOfFieldBody);
         if (folded) {
             rawFieldBody = unfoldHeader(rawFieldBody);
         }
         // We might get an invalid mail without a field name, don't crash on that.
         if (!rawType.isEmpty()) {
-            header = HeaderFactory::self()->createHeader(rawType);
+            header = HeaderFactory::createHeader(rawType);
         }
         if (!header) {
             //qWarning() << "Returning Generic header of type" << rawType;
@@ -2266,9 +2085,9 @@ void extractHeaderAndBody(const QByteArray &content, QByteArray &header, QByteAr
     }
 }
 
-Headers::Base::List parseHeaders(const QByteArray &head)
+QVector<Headers::Base*> parseHeaders(const QByteArray &head)
 {
-    Headers::Base::List ret;
+    QVector<Headers::Base*> ret;
     Headers::Base *h;
 
     QByteArray copy = head;
